#ifndef dp_5613_H
#define dp_5613_H

#define DPR_CODE_ATTR __cdecl

typedef unsigned char DPR_BYTE;
typedef unsigned short DPR_WORD;
typedef unsigned long DPR_DWORD;
typedef char DPR_STRING;

#define DP_ERR_TXT_SIZE  1024
#define DPR_SLAVE_DATA_SIZE 246
#define DPR_MAX_SLAVE_NR 128
#define DPR_DP_UNLOCK 0xFFFF

typedef struct
{
  DPR_BYTE data[DPR_SLAVE_DATA_SIZE];
}
DPR_SLAVE_IN_T;
typedef struct
{
  DPR_BYTE data[DPR_SLAVE_DATA_SIZE];
}
DPR_SLAVE_OUT_T;

typedef struct
{
  DPR_WORD slave_state;
}
DPR_SLAVE_INFO_T;
typedef struct
{
  DPR_WORD USIF_state;
}
DPR_MASTER_INFO_T;

typedef struct
{
  DPR_DWORD error_code;
}
DP_ERROR_T;
typedef struct
{
  DPR_WORD D_lock_in_slave_adr;
  DPR_WORD D_out_slave_adr;
}
DPR_CP5613_CTR_T;
typedef struct
{
  DPR_SLAVE_IN_T slave_in[DPR_MAX_SLAVE_NR];
  DPR_SLAVE_OUT_T slave_out[DPR_MAX_SLAVE_NR];
}
DPR_CP5613_PI_T;
typedef struct
{
  DPR_SLAVE_INFO_T slave_info[DPR_MAX_SLAVE_NR];
  DPR_MASTER_INFO_T master_info;
}
DPR_CP5613_INFO_WATCH_T;
typedef struct
{
  DPR_BYTE req_mask;
}
DPR_MASK_DATA_INT_T;
typedef struct
{
  DPR_MASK_DATA_INT_T input[DPR_MAX_SLAVE_NR];
}
DPR_CP5613_EF_T;
typedef struct
{
  DPR_CP5613_CTR_T ctr;
  DPR_CP5613_EF_T ef;
  DPR_CP5613_PI_T pi;
  DPR_CP5613_INFO_WATCH_T info_watch;
}
DPR_CP5613_DP_T;

#ifdef __cplusplus
extern "C"
{
#endif

  DPR_DWORD DPR_CODE_ATTR DP_start_cp (const DPR_STRING * cp_name,
				       const DPR_STRING * database,
				       DP_ERROR_T * error);

  DPR_DWORD DPR_CODE_ATTR DP_open (const DPR_STRING * cp_name,
				   DPR_DWORD * user_handle,
				   DP_ERROR_T * error);

  DPR_DWORD DPR_CODE_ATTR DP_get_pointer (DPR_DWORD user_handle,
					  DPR_DWORD timeout,
					  DPR_CP5613_DP_T volatile **dpr,
					  DP_ERROR_T * error);
  DPR_DWORD DPR_CODE_ATTR DP_set_mode (DPR_DWORD user_handle,
				       DPR_WORD mst_mode, DP_ERROR_T * error);

  DPR_DWORD DPR_CODE_ATTR DP_get_err_txt (DP_ERROR_T * error,
					  const DPR_STRING * language,
					  DPR_STRING text[DP_ERR_TXT_SIZE]);

  DPR_DWORD DPR_CODE_ATTR DP_release_pointer (DPR_DWORD user_handle,
					      DP_ERROR_T * error);

  DPR_DWORD DPR_CODE_ATTR DP_close (DPR_DWORD user_handle,
				    DP_ERROR_T * error);

  DPR_DWORD DPR_CODE_ATTR DP_reset_cp (const DPR_STRING * cp_name,
				       DP_ERROR_T * error);

  DPR_DWORD DPR_CODE_ATTR DP_init_sema_object (DPR_DWORD user_handle,
					       DPR_DWORD sema_type,
					       DPR_DWORD * sema_handle,
					       DP_ERROR_T * error);

  DPR_DWORD DPR_CODE_ATTR DP_delete_sema_object (DPR_DWORD user_handle,
						 DPR_DWORD sema_handle,
						 DP_ERROR_T * error);

#ifdef __cplusplus
}
#endif


#define DP_OK 0
#define DP_ERROR_CI 5

#define DP_OFFLINE 0
#define DP_STOP 1
#define DP_CLEAR 2
#define DP_AUTOCLEAR 3
#define DP_OPERATE 4

#define DPR_SLV_NOT_READY 0
#define DPR_SLV_READY 1

#define DP_OBJECT_TYPE_INPUT_CHANGE   0x1UL

#define DPR_DATA_INT_CLEAR_AND_UNMASK  0x00
#define DPR_DATA_INT_CLEAR_AND_MASK    0x0F
#define DPR_DATA_CHANGE                0xFF

#endif
